/*
Copyright (C) 2009  Ruan Strydom <ruan@jcell.co.za>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CPPPROPERTYCREATOR_H
#define CPPPROPERTYCREATOR_H

#include "propertcreator.h"
#include <QString>
#include <QList>
#include <KMessageBox>
#include <language/duchain/types/abstracttype.h>

namespace KDevelop
{
    class IDocument;
    class DUContext;
    class Declaration;
}

class PropertyNode;

class CppPropertyCreator : public PropertyCreator
{
public:
    bool parse (PropertyModel * m, QList<Declaration*> selection);
    void create (PropertyModel * m);
    
private:
    class CodeRange
    {
        public:
            CodeRange() : lineStart (0), lineEnd (0) {}
            CodeRange(const CodeRange& o) : name(o.name), lineStart (o.lineStart), lineEnd (o.lineEnd) {}
            CodeRange (const QString & n) : name (n), lineStart (0), lineEnd (0) {}
            CodeRange (const QString & n, int s, int e) : name (n), lineStart (s), lineEnd (e) {}
            virtual ~CodeRange() {}
            QString name;
            int lineStart;
            int lineEnd;
        };
    
    QList<CodeRange> m_ranges;
    CodeRange *getRange (const QString &name);
    int getRangeIndex (const QString &name);

    void addText(IDocument *doc, CodeRange *range, const QStringList &text, const QString &indent = "", int *lineCount = NULL);
    
    QString readerName (const QString &property);
    QString writerName (const QString &property);
    QString propertyName (const QString &property);
    QString propertyAlias (const QString &property);
    QString readerType (const QString & abstractType, const QString & resolvedType, AbstractType::WhichType type);
    QString writerType (const QString & abstractType, const QString & resolvedType, AbstractType::WhichType type);
};

#endif //CPPPROPERTYCREATOR_H

// kate: indent-mode cstyle; space-indent on; indent-width 4; replace-tabs on; 
